<?php
namespace Teknisa\FAI\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;

class Fai40000EstatisticaFaturamentoReport extends \Zeedhi\Framework\Controller\Report {
    
    protected $connection;
    
    public function __construct(ReportService $reportService, $qr2Strategy, $connection) {
        $this->qr2Strategy  = $qr2Strategy;
        $this->reportService  = $reportService;
        $this->connection = $connection;
    }

    protected $reportMapping = array (
        'FAI40000' =>
        array (
            'strategy'         => 'QR2',
            'parameterMapping' =>
                array (
                    'LCDFILIAL1'  => 'L_CDFILIAL1',
                    'LCDFILIAL2'  => 'L_CDFILIAL2',
                    'LCDFILIAL3'  => 'L_CDFILIAL3',
                    'LCDFILIAL4'  => 'L_CDFILIAL4',
                    'LCDFILIAL5'  => 'L_CDFILIAL5',
                    'LCDFILIAL6'  => 'L_CDFILIAL6',
                    'LCDFILIAL7'  => 'L_CDFILIAL7',
                    'LCDFILIAL8'  => 'L_CDFILIAL8',
                    'LCDFILIAL9'  => 'L_CDFILIAL9',
                    'LCDFILIAL10' => 'L_CDFILIAL10',

                    'LCDFILIFISC1'  => 'L_CDFILIFISC1',
                    'LCDFILIFISC2'  => 'L_CDFILIFISC2',
                    'LCDFILIFISC3'  => 'L_CDFILIFISC3',
                    'LCDFILIFISC4'  => 'L_CDFILIFISC4',
                    'LCDFILIFISC5'  => 'L_CDFILIFISC5',
                    'LCDFILIFISC6'  => 'L_CDFILIFISC6',
                    'LCDFILIFISC7'  => 'L_CDFILIFISC7',
                    'LCDFILIFISC8'  => 'L_CDFILIFISC8',
                    'LCDFILIFISC9'  => 'L_CDFILIFISC9',
                    'LCDFILIFISC10' => 'L_CDFILIFISC10',

                    'LCHAVE1'  => 'L_CDSERIE1',
                    'LCHAVE2'  => 'L_CDSERIE2',
                    'LCHAVE3'  => 'L_CDSERIE3',
                    'LCHAVE4'  => 'L_CDSERIE4',
                    'LCHAVE5'  => 'L_CDSERIE5',
                    'LCHAVE6'  => 'L_CDSERIE6',
                    'LCHAVE7'  => 'L_CDSERIE7',
                    'LCHAVE8'  => 'L_CDSERIE8',
                    'LCHAVE9'  => 'L_CDSERIE9',
                    'LCHAVE10' => 'L_CDSERIE10',

                    'LCDPAIS1'  => 'L_CDPAIS1',
                    'LCDPAIS2'  => 'L_CDPAIS2',
                    'LCDPAIS3'  => 'L_CDPAIS3',
                    'LCDPAIS4'  => 'L_CDPAIS4',
                    'LCDPAIS5'  => 'L_CDPAIS5',
                    'LCDPAIS6'  => 'L_CDPAIS6',
                    'LCDPAIS7'  => 'L_CDPAIS7',
                    'LCDPAIS8'  => 'L_CDPAIS8',
                    'LCDPAIS9'  => 'L_CDPAIS9',
                    'LCDPAIS10' => 'L_CDPAIS10',

                    'LSGESTADO1'  => 'L_SGESTADO1',
                    'LSGESTADO2'  => 'L_SGESTADO2',
                    'LSGESTADO3'  => 'L_SGESTADO3',
                    'LSGESTADO4'  => 'L_SGESTADO4',
                    'LSGESTADO5'  => 'L_SGESTADO5',
                    'LSGESTADO6'  => 'L_SGESTADO6',
                    'LSGESTADO7'  => 'L_SGESTADO7',
                    'LSGESTADO8'  => 'L_SGESTADO8',
                    'LSGESTADO9'  => 'L_SGESTADO9',
                    'LSGESTADO10' => 'L_SGESTADO10',

                    'LCDTIPOFATUR1'  => 'L_CDTIPOFATUR1',
                    'LCDTIPOFATUR2'  => 'L_CDTIPOFATUR2',
                    'LCDTIPOFATUR3'  => 'L_CDTIPOFATUR3',
                    'LCDTIPOFATUR4'  => 'L_CDTIPOFATUR4',
                    'LCDTIPOFATUR5'  => 'L_CDTIPOFATUR5',
                    'LCDTIPOFATUR6'  => 'L_CDTIPOFATUR6',
                    'LCDTIPOFATUR7'  => 'L_CDTIPOFATUR7',
                    'LCDTIPOFATUR8'  => 'L_CDTIPOFATUR8',
                    'LCDTIPOFATUR9'  => 'L_CDTIPOFATUR9',
                    'LCDTIPOFATUR10' => 'L_CDTIPOFATUR10',

                    'P_DTINICIALANT' => 'P_DTINICIALANT',
                    'P_DTFINALANT'   => 'P_DTFINALANT',
                    'P_DTINICIAL'    => 'P_DTINICIAL',
                    'P_DTFINAL'      => 'P_DTFINAL',
                    'P_CDTIPOOPER'   => 'P_CDTIPOOPER',
                    'P_TCDTIPOOPER'  => 'P_TCDTIPOOPER',
                    'P_OPORDENACAO'  => 'P_OPORDENACAO',
                    'VALORINICIAL'  => 'P_VALORINI',
                    'VALORFINAL'    => 'P_VALORFIN'
                ),
            ),
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        ini_set('default_socket_timeout', 40000);
        $row = $request->getRow();
        
        /*
        substitui o ponto(.) dos campos VALORINICICAL e VALORFINAL por vazio('')
        devido conflitos no banco de dados que contem formato PT-BR, gerando erro     
        */

        $row['VALORINICIAL'] = str_replace('.','',$row['VALORINICIAL']);
        $row['VALORFINAL'] = str_replace('.','',$row['VALORFINAL']);

        if($row['CHECK_CENTRALIZADORA'] === 'T'){
            $properties = array(
                'LCDFILIFISC' => $this->getAllFiliFisc()
            );
            if($row['CHECK_UNIDADE_APURACAO'] === 'T'){
                $properties['LCDFILIAL'] = $this->getAllFiliApurac('T');
            }
            $this->setProperty($row, $properties);
        } else {
            $properties = array(
                'LCDFILIAL' => $this->getAllFiliApurac(implode($row['CDFILIFISC'], '_'))
            );
            $this->setProperty($row, $properties);
        }
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
    }

    private function setProperty($row, $arrayMap){
        foreach(array_keys($arrayMap) as $key){
            foreach(array_values($this->reportMapping['FAI40000']['parameterMapping']) as $keyReport){
                $normalizedName = str_replace('_', '', $keyReport);
                if(strpos($normalizedName, $key) !== false){
                    $row[$keyReport] = $arrayMap[$key];
                }
            }
        }
    }
    
    private function getAllFiliFisc(){
        $sql = "SELECT LISTAGG(F.CDFILIFISC, '_') WITHIN GROUP (ORDER BY F.CDFILIFISC) CDFILIFISC 
                    FROM
                        FILIAL F,
                        FILIAL NM
                    WHERE
                        F.CDFILIFISC IS NOT NULL
                        AND F.CDFILIFISC = NM.CDFILIAL";
        $query = $this->connection->prepare($sql);
        $query->execute();
        return $query->fetch()['CDFILIFISC'];
    }
    
    private function getAllFiliApurac($filiFisc){
        $sql = "SELECT
                    LISTAGG(CDFILIAL, '_') WITHIN GROUP (ORDER BY CDFILIAL) CDFILIAL
                FROM
                    FILIAL
                WHERE
                    (
                        ( INSTR(:CDFILIFISC,CDFILIFISC) > 0 )
                        OR (:CDFILIFISC = 'T' )
                    )";   
        $query = $this->connection->prepare($sql);
        $query->execute(array(
            'CDFILIFISC' => $filiFisc
        ));
        return $query->fetch()['CDFILIAL'];
    }
}