<?php
namespace Teknisa\FAI\Modules\Tabelas\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ControlePedidos extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = 'pedidoserv_pessoa_cliente';

    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $newCode;
    protected $environment;
    protected $dataSourceManager;

    protected $lista_pedidos;

    const TABLE_LISTA = "LISTA";

    public function __construct($crudService, ParameterBag $parameterBag, $entityManager, $newCode, $environment, $dataSourceManager) {
        parent::__construct($crudService, $parameterBag);

        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
    }  

    /** 
     * generateOrCancelNF
     *
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     * 
     * @return Response $response
     */
    public function generateOrCancelNF (DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        $orders = $rows[0];
        $operationType = $rows[1];

        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            //Bloco de codigo feito para contornar incoerencia na procedure
            $procedureResultArray = array();

            if($operationType['OPERACAO'] === 'G'){
                $resultProcedure = $this->nfProcedure($orders, $operationType);
                $this->deleteLista($this->nrListaPedidos);

                array_push($procedureResultArray, $resultProcedure);
            } else {
                $resultProcedure = $this->nfProcedure($orders, $operationType);
                $this->deleteLista($this->nrListaPedidos);
                
                array_push($procedureResultArray, $resultProcedure);
            }

            $connection->commit();

            $response->addDataSet(new DataSet ('GERA_NF_MYORDER', $procedureResultArray));
        }catch(\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    /**
     * insertLista
     * Executa o insert da lista
     *
     * @param Array $listaPedidos
     *
     * @return int|string
     */
    private function insertLista($listaPedidos) {
        $dataSourceName = strtolower(self::TABLE_LISTA);
        $nrlista = $this->newCode->getCodeByTableName(self::TABLE_LISTA, $this->environment->getNrOrg());
        $rows = array();
        foreach ($listaPedidos as $pedido) {
            $row = array(
                "NRLISTA" => $nrlista,
                "CDLISTA" => $pedido['NRPEDIDOSERV'],
                "__is_new" => true
            );
            array_push($rows, $row);
        }
        $dataSet = new DataSet($dataSourceName, $rows);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");

        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        $this->crudService->save($dataSourceName, $rowsToSave);
        return $nrlista;
    }

    /**
     * deleteLista
     * Executa o delete da lista
     *
     * @param int|string $nrLista
     *
     * @return Void
     */
    private function deleteLista($nrlista) {
        $dataSourceName = strtolower(self::TABLE_LISTA);
        $conditions = array(
            array(
                "columnName" => "NRLISTA",
                "operator"   => '=',
                "value"      => $nrlista
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        $this->dataSourceManager->delete($dataSet);
    }

    /**
     * formatResultOfProcedure
     * Formata result da procedure adicionando informacoes relevantes
     * sobre o pedido
     *
     * @param Array $result
     * @param Array $orders
     *
     * @return Array
     */
    private function formatResultOfProcedure($result, $orders) {
        $result['ID'] = array();

        foreach ($orders as $order) {
            array_push($result['ID'], $order['NRPEDIDOSERV']);
        }

        return $result;
    }

    /**
     * nfProcedure
     * Executa a procedure no BD e retorna resultado
     *
     * @param Array $orders
     * @param Array $operationType
     *
     * @return Array
     */
    private function nfProcedure($orders, $operationType) {
        $this->nrListaPedidos = $this->insertLista($orders); 
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_NF_MYORDER');

        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->environment->getNrOrg(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAPEDIDO', Param::PARAM_INPUT, $this->nrListaPedidos, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, $operationType['OPERACAO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 400));
        
        $result = $procedure->execute();

        return $this->formatResultOfProcedure($result, $orders);
    }
}