<?php
namespace Teknisa\FAI\Modules\Tabelas\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;

class Fai04600TabelaDeCorrecaoParaCompensacao extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $environment;
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName;
    protected $parameterBagColumns;

    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $util) {

        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
        $this->util = $util;
    }
    public function getPrecoServico($request, $response) {
        try {
            $row= $request->getDataset()->getRows()[0];
            $conditions = array(
                array(
                    'columnName'   => 'CDFILIAL',
                    'operator'     => '=',
                    'value'        => $row['CDFILIAL']
                ),
                array(
                    'columnName'   => 'CDSERVFILI',
                    'operator'     => '=',
                    'value'        => $row['CDSERVFILI']
                ),
                array(
                    'columnName'   => 'CDCLIENTE',
                    'operator'     => '=',
                    'value'        => $row['CDCLIENTE']
                ),
                array(
                    'columnName'   => 'NRORG',
                    'operator'     => '=',
                    'value'        => $this->environment->getNrorgTrab()
                )
            );
            $this->parameterBagColumns = array("CDFILIAL", "CDSERVFILI", "CDCLIENTE", "NRORG");
            $filterCriteria = new FilterCriteria('fai04600_findPrecoServico', $conditions);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
            $response->addDataset($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function getCliePrinc($request, $response) {
        try {
            $row= $request->getDataset()->getRows()[0];
            $conditions = array(
                array(
                    'columnName'   => 'IDCLIENPRIN',
                    'operator'     => '=',
                    'value'        => 'S'
                ),
                array(
                    'columnName'   => 'CDFILIAL',
                    'operator'     => '=',
                    'value'        => $row['CDFILIAL']
                ),
                array(
                    'columnName'   => 'NRORG',
                    'operator'     => '=',
                    'value'        => $this->environment->getNrorgTrab()
                )
            );
            $filterCriteria = new FilterCriteria('clienfilial', $conditions);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            $response->addDataset($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveParamCompensacao($request, $response) {
        try {
            $row = $request->getDataset()->getRows()[0];
            $this->dataSourceName = "paramcompensacao";
            if ($row["__is_new"] && $this->validateUnidadeServico($row)) {
                throw new \Exception("Impossível adicionar. Registro já existe.", 1);
            } else {
                $rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);
                $callBack = array(
                    "NRPARAMCOMPENSACAO" => intval($rowsToSave[0]["NRPARAMCOMPENSACAO"])
                );
                $newDataSet = new DataSet($this->dataSourceName, $callBack);
                $response->addDataset($newDataSet);
                $this->crudService->save($this->dataSourceName, $rowsToSave);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function validateUnidadeServico($row) {
        try {
            $conditions = array(
                array(
                    "columnName" => "CDFILIAL",
                    "operator" => "=",
                    "value" => $row["CDFILIAL"]
                ),
                array(
                    "columnName" => "CDSERVFILI",
                    "operator" => "=",
                    "value" => $row["CDSERVFILI"]
                ),
                array(
                    "columnName" => "DTVIGENCIA",
                    "operator" => "=",
                    "value" => $row["DTVIGENCIA"]
                )
            );

            $dataSet = $this->crudService->getFilteredDataSet($this->dataSourceName, $conditions);
            return !empty($dataSet->getRows());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }

    public function saveAgrupServ(DTO\Request $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "servicocompensacao";
            $row = $request->getDataSet()->getRows()[0];

            if ($row["__is_new"]) {
                $this->util->validateDuplicatedRecords($row, $this->dataSourceName, array("CDFILIAL", "CDSERVFILI", "NRPARAMCOMPENSACAO"));
            }

            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}
